@@ -14,7 +14,7 @@
 # limitations under the License.
 #
 import re
-import sys
+
 from ocp_tessellate import PartGroup
 from ocp_tessellate.convert import (
     tessellate_group,
@@ -23,6 +23,7 @@ from ocp_tessellate.convert import (
     to_assembly,
     mp_get_results,
     is_topods_shape,
+    is_cadquery,
     is_vector,
 )
 from ocp_tessellate.utils import numpy_to_buffer_json, Timer, Color
@@ -34,7 +35,6 @@ from ocp_tessellate.ocp_utils import (
     is_cadquery_assembly,
     is_cadquery_sketch,
     is_build123d,
-    is_build123d_assembly,
     is_toploc_location,
 )
 
@@ -46,7 +46,7 @@ from ocp_tessellate.cad_objects import (
     OCP_Part,
     OCP_Vertices,
 )
-from ocp_tessellate.convert import to_assembly, conv
+from ocp_tessellate.convert import to_assembly
 import ocp_tessellate.convert as oc
 
 from .config import (
@@ -55,38 +55,35 @@ from .config import (
     workspace_config,
     combined_config,
     get_default,
-    status,
-    set_viewer_config,
     Camera,
     Collapse,
     check_deprecated,
 )
-from .comms import send_data, MessageType
+from .comms import send_backend, send_data
 from .colors import *
 
+from .persistence import modify_copyreg
+
 __all__ = ["show", "show_object", "reset_show", "show_all", "show_clear"]
 
 OBJECTS = {"objs": [], "names": [], "colors": [], "alphas": []}
 
-FIRST_CALL = True
 LAST_CALL = "other"
 
+modify_copyreg()
+
 
 def _tessellate(
     *cad_objs, names=None, colors=None, alphas=None, progress=None, **kwargs
 ):
-    global FIRST_CALL
-
     if workspace_config().get("_splash"):
         conf = combined_config(use_status=False)
+        reset_camera = Camera.RESET
     else:
         conf = combined_config(use_status=True)
-        if FIRST_CALL:
-            conf["reset_camera"] = Camera.RESET.value
-            FIRST_CALL = False
-        else:
-            reset_camera = conf.get("reset_camera", Camera.RESET)
-            conf["reset_camera"] = reset_camera.value
+        reset_camera = conf.get("reset_camera", Camera.RESET)
+
+    conf["reset_camera"] = reset_camera.value
 
     collapse = conf.get("collapse", Collapse.LEAVES)
     conf["collapse"] = collapse.value
@@ -171,6 +168,8 @@ def _tessellate(
             print(f"Setting {k} can only be set in VSCode config")
 
         elif v is not None:
+            if k == "reset_camera" and params.get("_splash") == True:
+                continue
             params[k] = v
 
     parallel = preset("parallel", params.get("parallel"))
@@ -189,7 +188,7 @@ def _tessellate(
             init_pool()
             keymap.reset()
 
-        instances, shapes, states = tessellate_group(
+        instances, shapes, states, mapping = tessellate_group(
             part_group, params, progress, params.get("timeit")
         )
 
@@ -208,16 +207,24 @@ def _tessellate(
 
     # add global bounding box
     shapes["bb"] = bb
-    return instances, shapes, states, params, part_group.count_shapes()
+
+    return instances, shapes, states, params, part_group.count_shapes(), mapping
 
 
-def _convert(*cad_objs, names=None, colors=None, alphas=None, progress=None, **kwargs):
+def _convert(
+    *cad_objs,
+    names=None,
+    colors=None,
+    alphas=None,
+    progress=None,
+    **kwargs,
+):
     timeit = preset("timeit", kwargs.get("timeit"))
 
     if progress is None:
         progress = Progress([c for c in "-+c"])
 
-    instances, shapes, states, config, count_shapes = _tessellate(
+    instances, shapes, states, config, count_shapes, mapping = _tessellate(
         *cad_objs,
         names=names,
         colors=colors,
@@ -225,6 +232,7 @@ def _convert(*cad_objs, names=None, colors=None, alphas=None, progress=None, **k
         progress=progress,
         **kwargs,
     )
+
     if config.get("dark") is not None:
         config["theme"] = "dark"
     elif config.get("orbit_control") is not None:
@@ -237,16 +245,14 @@ def _convert(*cad_objs, names=None, colors=None, alphas=None, progress=None, **k
         config["explode"] = kwargs["explode"]
 
     with Timer(timeit, "", "create data obj", 1):
-        data = {
+        return {
             "data": numpy_to_buffer_json(
                 dict(instances=instances, shapes=shapes, states=states)
             ),
             "type": "data",
             "config": config,
             "count": count_shapes,
-        }
-
-    return data
+        }, mapping
 
 
 class Progress:
@@ -265,7 +271,7 @@ def align_attrs(attr_list, length, default, tag, explode=True):
     if attr_list is None:
         return [None] * length if explode else None
     elif len(attr_list) < length:
-        print(f"Too view {tag}, using defaults to fill")
+        print(f"Too few {tag}, using defaults to fill")
         return list(attr_list) + [default] * (length - len(attr_list))
     elif len(attr_list) > length:
         print(f"Too many {tag}, trimming to length {length}")
@@ -283,6 +289,7 @@ def show(
     progress="-+c",
     glass=None,
     tools=None,
+    measure_tools=None,
     tree_width=None,
     axes=None,
     axes0=None,
@@ -343,6 +350,7 @@ def show(
     Valid keywords to configure the viewer (**kwargs):
     - UI
         glass:                   Use glass mode where tree is an overlay over the cad object (default=False)
+        measure_tools:           Show measure tools (default=False)
         tools:                   Show tools (default=True)
         tree_width:              Width of the object tree (default=240)
 
@@ -398,7 +406,7 @@ def show(
         render_joints:           Render build123d joints (default=False)
         parallel:                Tessellate objects in parallel (default=False)
         show_parent:             Render parent of faces, edges or vertices as wireframe
-        helper_scale:              Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
+        helper_scale:            Scale of rendered helpers (locations, axis, mates for MAssemblies) (default=1)
 
     - Debug
         debug:                   Show debug statements to the VS Code browser console (default=False)
@@ -406,10 +414,6 @@ def show(
     """
     global LAST_CALL
 
-    # if sys.gettrace() is not None and not _force_in_debug:
-    #     print("\nshow and show_object are ignored in debugging sessions\n")
-    #     return
-
     kwargs = {
         k: v
         for k, v in locals().items()
@@ -428,9 +432,23 @@ def show(
 
     kwargs = check_deprecated(kwargs)
 
+    if kwargs.get("grid") is not None:
+        if isinstance(kwargs["grid"], bool):
+            kwargs["grid"] = [kwargs["grid"]] * 3
+
     timeit = preset("timeit", timeit)
 
-    names = align_attrs(names, len(cad_objs), None, "names", explode=False)
+    if measure_tools is None:
+        measure_tools = get_default("measure_tools")
+        if measure_tools is None:
+            conf = workspace_config()
+            measure_tools = conf["measure_tools"]
+    kwargs["measure_tools"] = measure_tools
+
+    if measure_tools and kwargs.get("collapse") is None:
+        kwargs["collapse"] = Collapse.ROOT
+
+    names = align_attrs(names, len(cad_objs), None, "names", explode=measure_tools)
 
     # Handle colormaps
 
@@ -464,7 +482,7 @@ def show(
     progress = Progress([] if progress is None else [c for c in progress])
 
     with Timer(timeit, "", "overall"):
-        data = _convert(
+        t, mapping = _convert(
             *cad_objs,
             names=names,
             colors=colors,
@@ -473,13 +491,16 @@ def show(
             **kwargs,
         )
 
-    if not _force_in_debug:
-        LAST_CALL = "show"
-    else:
-        LAST_CALL = "other"
+        if not _force_in_debug:
+            LAST_CALL = "show"
+        else:
+            LAST_CALL = "other"
 
     with Timer(timeit, "", "send"):
-        return send_data(data, port=port, timeit=timeit)
+        send_data(t, port=port, timeit=timeit)
+
+    if measure_tools:
+        send_backend({"model": mapping}, port=port, timeit=timeit)
 
 
 def reset_show():
@@ -498,6 +519,7 @@ def show_object(
     progress="-+c",
     glass=None,
     tools=None,
+    measure_tools=None,
     tree_width=None,
     axes=None,
     axes0=None,
@@ -560,6 +582,7 @@ def show_object(
     Valid keywords to configure the viewer (**kwargs):
     - UI
         glass:                   Use glass mode where tree is an overlay over the cad object (default=False)
+        measure_tools:           Show measure tools (default=False)
         tools:                   Show tools (default=True)
         tree_width:              Width of the object tree (default=240)
 
@@ -674,10 +697,13 @@ def show_clear():
     send_data(data)
 
 
-def show_all(variables=None, exclude=None, **kwargs):
+def show_all(variables=None, exclude=None, force=False, **kwargs):
     import inspect
 
-    global FIRST_CALL, LAST_CALL
+    global LAST_CALL
+
+    if force:
+        LAST_CALL = "other"
 
     if LAST_CALL == "show":
         LAST_CALL = "other"
@@ -708,8 +734,8 @@ def show_all(variables=None, exclude=None, **kwargs):
             if hasattr(obj, "locations") and hasattr(obj, "local_locations"):
                 obj = obj.locations
 
-            if hasattr(obj, "to_location"):
-                obj = obj.to_location()
+            if hasattr(obj, "local_coord_system"):
+                obj = obj.location
 
             if (
                 (
@@ -729,6 +755,11 @@ def show_all(variables=None, exclude=None, **kwargs):
                     and len(obj) > 0
                     and hasattr(obj[0], "wrapped")
                 )
+                or (
+                    hasattr(obj, "wrapped")
+                    and hasattr(obj, "position")
+                    and hasattr(obj, "direction")
+                )
             ):
                 objects.append(obj)
                 names.append(name)
@@ -746,9 +777,6 @@ def show_all(variables=None, exclude=None, **kwargs):
                 objects.append(pg)
                 names.append(name)
 
-    if FIRST_CALL:
-        kwargs["reset_camera"] = Camera.RESET
-
     if len(objects) > 0:
         show(
             *objects,
@@ -757,6 +785,5 @@ def show_all(variables=None, exclude=None, **kwargs):
             _force_in_debug=True,
             **kwargs,
         )
-        FIRST_CALL = False
     else:
         show_clear()
